require 'zip/zip'

class MultipleFilesUploadController < ApplicationController
  unloadable

  ZIP_FILENAMES_ENCODING = 'cp866'
  ZIP_FILENAMES_ENCODING_FALLBACK = 'cp850'

  before_filter :setup_environment

  # container_type
  # 'Issue'
  # 'Wiki'
  # 'Message'
	def index
    deny_access unless @allow_upload
  end

  def download
    retrieve_attachments_list
  end

  def download_zip
    retrieve_attachments_list
    attachments=filter_attachments_list(params[:files_to_download])
    zip_stream=process_attachments_export(attachments)
    send_data(zip_stream, :type => 'application/zip', :filename => get_archive_filename)
  end

	def upload
    if @allow_upload
      attr={}
      attr[:attachments]={'1' => {'file' => params[:Filedata], 'description' => ''}}
      case @container_type
        when 'Issue'
          journal=@container.init_journal(User.current,'')
          Attachment.attach_files(@container, attr[:attachments])
          #attachments[:files].each {|a| journal.details << JournalDetail.new(:property => 'attachment', :prop_key => a.id, :value => a.filename)}
          if @container.save
            journal
          end
        when 'Wiki'
          Attachment.attach_files(@container, attr[:attachments])
        when 'Message'
          Attachment.attach_files(@container, attr[:attachments])
      end

      render :text => params[:Filename]
    end
  end

private

	def setup_environment
		retrieve_objects_from_params
		unless @container
			deny_access
		end
	end

	def retrieve_objects_from_params
		@container=nil
    @allow_upload=false
		return @container if params[:container_type].blank? || params[:container_id].blank?

		obj=nil
		case params[:container_type]
			when 'Issue'
				obj=Issue.find_by_id(params[:container_id].to_i)
				if obj
					obj=nil unless (User.current.allowed_to?(:view_issues,obj.project) && obj.visible?(User.current))
          @allow_upload=true if (obj && User.current.allowed_to?(:add_issue_notes,obj.project))
				end
				@container_back_url=url_for(:controller => 'issues', :action => 'show', :id => obj.id.to_s) if obj
			when 'Wiki'
				obj=WikiPage.find_by_id(params[:container_id].to_i)
				if obj
					obj=nil unless User.current.allowed_to?(:view_wiki_pages,obj.project)
          @allow_upload=true if (obj && User.current.allowed_to?(:edit_wiki_pages,obj.project))
				end
				@container_back_url=url_for(:controller => 'wiki', :action => 'show', :id => obj.title, :project_id => obj.project) if obj
			when 'Message'
				obj=Message.find_by_id(params[:container_id].to_i)
				if obj
          obj=nil unless User.current.allowed_to?(:view_messages,obj.project)
          @allow_upload=true if (obj && obj.editable_by?(User.current))
				end
				@container_back_url=create_link_to_message(obj) if obj
		end

		@container=obj
		@container_type=(obj ? params[:container_type] : '')

		return obj
	end

  def create_link_to_message(msg)
	  link_to_str = { :controller => 'messages', :action => 'show', :board_id => msg.board }
	  if msg.root && msg.root != msg
		  link_to_str[:id]=msg.root
		  link_to_str[:r]=msg
		  link_to_str[:anchor]="message-#{msg.id}"
	  else
		  link_to_str[:id]=msg
	  end
	  url_for(link_to_str)
  end

  def retrieve_attachments_list
    @attachments=[]
    if @container.attachments
      filenames_hash={}
      @container.attachments.each do |a|
        filenames_hash[a.filename.downcase] = [] if filenames_hash[a.filename.downcase].blank?
        filenames_hash[a.filename.downcase] << a
      end

      filenames_hash.each_value do |a|
        a.sort! do |v1,v2|
          v2.created_on <=> v1.created_on
        end
      end

      filenames_a=filenames_hash.to_a.sort do |v1,v2|
        v1[0] <=> v2[0]
      end

      filenames_a.each do |f|
        f[1].each_with_index do |a,idx|
          @attachments << { :object => a, :idx => idx}
        end
      end
      @attachments
    end
  end

  def process_attachments_export(files_to_download)
    zip_file=Zip::ZipOutputStream::write_buffer do |zip_stream|
      files_to_download.each do |attach|
        file=File.open(attach[:object].diskfile,'rb')
        zip_stream.put_next_entry(create_zip_filename(attach),nil,nil,Zip::ZipEntry::DEFLATED,Zlib::BEST_COMPRESSION)
        zip_stream.write(file.read)
      end
    end

    zip_file.string
  end

  def get_archive_filename
    str=''
    str << "#{@container_type}_#{@container.id.to_s}_files.zip"
    str
  end

  def filter_attachments_list(ids_str_array)
    attachments_new=[]
    ids_str_array.each do |a_id|
      attach=@attachments.detect do |a|
        true if a_id.to_i == a[:object].id
      end
      attachments_new << attach if attach
    end
    attachments_new
  end

  def create_zip_filename(attach)

    new_name=''
    if attach[:idx] == 0
      new_name << attach[:object].filename
    else
      md=attach[:object].filename.match(/^(.+)\.([^ .]+?)$/)
      if md
        new_name << "#{md[1]}(%02i).#{md[2]}" % [attach[:idx].to_i]
      else
        new_name << "#{attach[:object].filename}(%02i)" % [attach[:idx].to_i]
      end
    end

    begin
      Iconv.iconv(ZIP_FILENAMES_ENCODING,'utf-8', new_name).join
    rescue
      begin
        Iconv.iconv(ZIP_FILENAMES_ENCODING_FALLBACK,'utf-8', new_name).join
      rescue
        new_name
      end
    end
  end
end

